import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * This is the low-level class used to read a stream of data off a file
 * stored on the webserver.  In reality, most people will not use this
 * class directly, but will instead create it and use it to create
 * an ObjectInputStream or DataInputStream.
 *
 * <P>The opposite of this class is the WebsaverOutputStream.
 *
 * @author Brad Fitzpatrick &lt;bradfitz@bradfitz.com&gt;
 *
 * @see WebsaverOutputStream
 * @see java.io.ObjectInputStream
 * @see java.io.DataInputStream
 *
 */
public class WebsaverInputStream extends InputStream
{
    private String streamData;
    private int bytesRead = 0;

    /**
     * Constructor for a WebsaverInputStream when the requested file was saved with
     * a password required for reading.
     *
     * @param wsConn    A previously created WebsaverConnection object, used to
     *                  communicate with the server.
     *            
     * @param filename  The filename on the server to read from.  Only alphanumerics, 
     *                  underscore, period, and hyphen are allowed in the filename.
     *
     * @param password  The password to read this file, if needed.  An exception will
     *                  be generated if you do not supply the correct password.
     */
    public WebsaverInputStream (WebsaverConnection wsConn, String filename, String password) throws IOException
    {
	// send the request to the server using WebsaverConnection
	Hashtable req = new Hashtable();
	req.put("mode", "readfile");
	req.put("filename", filename);
	if (password != null)
	    req.put("password", password);
	Hashtable res = wsConn.sendRequest(req);
	
	// throw an exception if the server returned an error or 
	// didn't return success
	String success = (String) res.get("success");
	if (success == null || !success.equals("OK")) {
	    if (success != null)
		throw(new IOException("Error: " + (String)res.get("errmsg")));
	    else
		throw(new IOException("Websaver server is not working correctly."));
	}

	// assuming things are okay, hold onto the streamdata for the user to read
	streamData = (String) res.get("streamdata");
    }

    /**
     * Constructor for a WebsaverInputStream when the requested file was saved with
     * no password or w/ public-read access enabled.
     *
     * @param wsConn    A previously created WebsaverConnection object, used to
     *                  communicate with the server.
     *
     * @param filename  The filename on the server to read from.  Only alphanumerics, 
     *                  underscore, period, and hyphen are allowed in the filename.
     */
    public WebsaverInputStream (WebsaverConnection wsConn, String filename) throws IOException
    {
	this(wsConn, filename, null);
    }

    /**
     * Returns the number of bytes left in the stream.
     *
     * @return The number of bytes left in the stream, or 0 if the stream is empty.
     */
    public int available ()
    {
	if (streamData == null)
	    return 0;
	return (streamData.length()-bytesRead);
    }

    /**
     * Returns the next byte in the stream.
     *
     * @return The next byte in the stream, or -1 if the end is reached.
     */
    public int read () 
    {
	if (streamData == null || bytesRead >= streamData.length())
	    return -1;
	
	return (int) streamData.charAt(bytesRead++);
    }

    /**
     * Closes the connection and frees any resources associated with it.
     *
     * @return None
     */
    public void close ()
    {
	streamData = null;
	bytesRead = 0;
    }

}
