import java.io.OutputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * This is the low-level class used to write a stream of data into a file
 * stored on the webserver.  In reality, most people will not use this
 * class directly, but will instead create it and use it to create
 * an ObjectOutputStream or DataOutputStream.
 *
 * <P>The opposite of this class is the WebsaverInputStream.
 *
 * @author Brad Fitzpatrick &lt;bradfitz@bradfitz.com&gt;
 * @see WebsaverInputStream
 * @see java.io.ObjectOutputStream
 * @see java.io.DataOutputStream
 *
 */
public class WebsaverOutputStream extends OutputStream
{
    private StringBuffer writtenData;
    private String url, filename, password;
    private WebsaverConnection wsConn;
    private boolean publicReadable;

    /**
     * Constructor for a WebsaverOutputStream when the requested file was saved with
     * a password required for writing, or to protect a new file w/ a password.
     *
     * @param wsConn    A previously created WebsaverConnection object, used to
     *                  communicate with the server.
     *       
     * @param filename  The filename on the server to read from.  Only alphanumerics, 
     *                  underscore, period, and hyphen are allowed in the filename.
     *
     * @param password  The password required to write to this file, if the file exists, and
     *                  also the password required to read this file if publicReadable is not
     *                  true.
     *
     * @param publicReadable  A boolean that was set, indicates users only need a password to
     *                        overwrite this file, but they may read from it without a password.
     */
    public WebsaverOutputStream (WebsaverConnection wsConn, String filename, String password, boolean publicReadable)
    {
	this.wsConn = wsConn;
	this.url = url;
	this.filename = filename;
	this.password = password;
	this.publicReadable = publicReadable;

	writtenData = new StringBuffer();
    }

    /**
     * Constructor for a WebsaverOutputStream for a new file without a password or for
     * an existing file on the server that is not password protected.
     *
     * @param wsConn    A previously created WebsaverConnection object, used to
     *                  communicate with the server.
     *
     * @param filename  The filename on the server to read from.  Only alphanumerics, 
     *                  underscore, period, and hyphen are allowed in the filename.
     *
     * @param password  The password required to write to this file, if the file exists, and
     *                  also the password required to read this file if publicReadable is not
     *                  true.
     *
     * @param publicReadable  A boolean that was set, indicates users only need a password to
     *                        overwrite this file, but they may read from it without a password.
     */
    public WebsaverOutputStream (WebsaverConnection wsConn, String filename)
    {
	this(wsConn, filename, null, true);
    }

    /**
     * Writes the next byte to the stream.
     *
     * @return None
     */
    public void write (int b) 
    {
	if (b < 0) b += 256;
	writtenData.append((char)b);
    }

    /**
     * <B>Important:</B> The data written to the stream is only saved once the stream is
     * closed.  Although close() should be automatically called when the object is about
     * ready to go out of scope, it is best to call this method explicitly to ensure your
     * data is saved.  Calling close() on any higher-level output class that uses this
     * class should be sufficient.
     *
     * @return None
     */
    public void close () throws IOException
    {
	Hashtable req = new Hashtable();
	req.put("mode", "writefile");
	req.put("filename", filename);
	if (password != null)
	    req.put("password", password);
	req.put("streamdata", writtenData.toString());
	if (publicReadable)
	    req.put("publicread", "1");

	Hashtable res = wsConn.sendRequest(req);
	String success = (String) res.get("success");
	if (success == null || !success.equals("OK")) {
	    if (success != null)
		throw(new IOException("Error: " + (String)res.get("errmsg")));
	    else
		throw(new IOException("Websaver server is not working correctly."));
	}

	writtenData.setLength(0);
    }

}
